==> sugarpowder-0.0.2.tar.gz: sugarpowder-0.0.2/src/sugarpowder/pipes.py <==
from __future__ import annotations

"""Library allowing a sh like infix syntax using pipes."""

__author__ = "Julien Palard <julien@python.org>"
__version__ = "2.0"
__credits__ = """Jérôme Schneider for teaching me the Python datamodel,
and all contributors."""

import functools
import itertools
import socket
import sys
from contextlib import closing
from collections import deque
import builtins
from typing import Any, Callable, Generator, Generic, Iterable, ParamSpec, TypeVar

T = TypeVar("T")
P = ParamSpec("P")


class Pipe(Generic[P, T]):
    """
    Represent a Pipeable Element :
    Described as :
    first = Pipe(lambda iterable: next(iter(iterable)))
    and used as :
    print [1, 2, 3] | first
    OR print [1, 2, 3] >> first
    printing 1

    Or represent a Pipeable Function :
    It's a function returning a Pipe
    Described as :
    select = Pipe(lambda iterable, pred: (pred(x) for x in iterable))
    and used as :
    print [1, 2, 3] | select(lambda x: x * 2)
    OR print >> select(lambda x: x * 2)
    # 2, 4, 6
    """

    def __init__(self, function: Callable[..., T]):
        self.function = function
        functools.update_wrapper(self, function)

    def __call__(self, *args: P.args, **kwargs: P.kwargs) -> Pipe[Callable[..., Any], T]:
        return Pipe(lambda iterable, *args2, **kwargs2: self.function(iterable, *args, *args2, **kwargs, **kwargs2))

    def __ror__(self, other) -> T:
        return self.function(other)

    def __rrshift__(self, other: P) -> T:
        return self.function(other)


@Pipe
def take(iterable: Iterable[T], qte: int) -> Generator[T, Any, None]:
    "Yield qte of elements in the given iterable."
    for item in iterable:
        if qte > 0:
            qte -= 1
            yield item
        else:
            return


@Pipe
def tail(iterable: Iterable[T], qte: int) -> deque[T]:
    "Yield qte of elements in the given iterable."
    return deque(iterable, maxlen=qte)


@Pipe
def skip(iterable, qte):
    "Skip qte elements in the given iterable, then yield others."
    for item in iterable:
        if qte == 0:
            yield item
        else:
            qte -= 1


@Pipe
def dedup(iterable, key=lambda x: x):
    """Only yield unique items. Use a set to keep track of duplicate data."""
    seen = set()
    for item in iterable:
        dupkey = key(item)
        if dupkey not in seen:
            seen.add(dupkey)
            yield item


@Pipe
def uniq(iterable, key=lambda x: x):
    """Deduplicate consecutive duplicate values."""
    iterator = iter(iterable)
    try:
        prev = next(iterator)
    except StopIteration:
        return
    yield prev
    prevkey = key(prev)
    for item in iterator:
        itemkey = key(item)
        if itemkey != prevkey:
            yield item
        prevkey = itemkey


enumerate = Pipe(builtins.enumerate)


@Pipe
def permutations(iterable, r=None):
    # permutations('ABCD', 2) --> AB AC AD BA BC BD CA CB CD DA DB DC
    # permutations(range(3)) --> 012 021 102 120 201 210
    for x in itertools.permutations(iterable, r):
        yield x


@Pipe
def netcat(to_send, host, port):
    """Send and receive bytes over TCP."""
    with closing(socket.socket(socket.AF_INET, socket.SOCK_STREAM)) as s:
        s.connect((host, port))
        for data in to_send | traverse:
            s.send(data)
        while 1:
            data = s.recv(4096)
            if not data:
                break
            yield data


@Pipe
def traverse(args):
    if isinstance(args, (str, bytes)):
        yield args
        return
    for arg in args:
        try:
            yield from arg | traverse
        except TypeError:
            # not iterable --- output leaf
            yield arg


@Pipe
def tee(iterable):
    for item in iterable:
        sys.stdout.write(repr(item) + "\n")
        yield item


@Pipe
def select(iterable: Iterable[T], selector: Callable[[T], Any]) -> "builtins.map[T]":
    return builtins.map(selector, iterable)


map = select
""


@Pipe
def where(iterable: Iterable[T], predicate: Callable[[T], bool]) -> Generator[T, None, None]:
    return (x for x in iterable if predicate(x))


filter = where


@Pipe
def take_while(iterable, predicate):
    return itertools.takewhile(predicate, iterable)


@Pipe
def skip_while(iterable, predicate):
    return itertools.dropwhile(predicate, iterable)


@Pipe
def groupby(iterable, keyfunc):
    return itertools.groupby(sorted(iterable, key=keyfunc), keyfunc)


@Pipe
def sort(iterable: Iterable[T], key=None, reverse=False) -> list[T]:  # pylint: disable=redefined-outer-name
    return sorted(iterable, key=key, reverse=reverse)


@Pipe
def reverse(iterable):
    return reversed(iterable)


@Pipe
def t(iterable, y):
    if hasattr(iterable, "__iter__") and not isinstance(iterable, str):
        return iterable + type(iterable)([y])
    return [iterable, y]


@Pipe
def transpose(iterable):
    return list(zip(*iterable))


@Pipe
def tolist(iterable: Iterable[T]) -> list[T]:
    return list(iterable)


@Pipe
def toset(iterable: Iterable[T]) -> set[T]:
    return set(iterable)


chain = Pipe(itertools.chain.from_iterable)
chain_with = Pipe(itertools.chain)
islice = Pipe(itertools.islice)
izip = Pipe(zip)

if __name__ == "__main__":
    import doctest

    doctest.testfile("README.md")

==> sugarpowder-1.0.0.tar.gz: sugarpowder-1.0.0/src/sugarpowder/pipes.py <==
from __future__ import annotations

"""Library allowing a sh like infix syntax using pipes."""

__author__ = "Julien Palard <julien@python.org>"
__version__ = "2.0"
__credits__ = """Jérôme Schneider for teaching me the Python datamodel,
and all contributors."""

import functools
import itertools
import socket
import sys
from contextlib import closing
from collections import deque
import builtins
from typing import Any, Callable, Generator, Generic, Iterable, ParamSpec, TypeVar

T = TypeVar("T")
S = TypeVar("S")
P = ParamSpec("P")


class Pipe(Generic[P, T]):
    """
    Represent a Pipeable Element :
    Described as :
    first = Pipe(lambda iterable: next(iter(iterable)))
    and used as :
    print [1, 2, 3] | first
    OR print [1, 2, 3] >> first
    printing 1

    Or represent a Pipeable Function :
    It's a function returning a Pipe
    Described as :
    select = Pipe(lambda iterable, pred: (pred(x) for x in iterable))
    and used as :
    print [1, 2, 3] | select(lambda x: x * 2)
    OR print >> select(lambda x: x * 2)
    # 2, 4, 6
    """

    def __init__(self, function: Callable[..., T]):
        self.function = function
        functools.update_wrapper(self, function)

    def __call__(self, *args: P.args, **kwargs: P.kwargs) -> Pipe[Callable[..., Any], T]:
        return Pipe(lambda iterable, *args2, **kwargs2: self.function(iterable, *args, *args2, **kwargs, **kwargs2))

    def __ror__(self, other) -> T:
        return self.function(other)

    def __rrshift__(self, other: P) -> T:
        return self.function(other)


@Pipe
def take(iterable: Iterable[T], qte: int) -> Generator[T, Any, None]:
    "Yield qte of elements in the given iterable."
    for item in iterable:
        if qte > 0:
            qte -= 1
            yield item
        else:
            return


@Pipe
def tail(iterable: Iterable[T], qte: int) -> deque[T]:
    "Yield qte of elements in the given iterable."
    return deque(iterable, maxlen=qte)


@Pipe
def skip(iterable, qte):
    "Skip qte elements in the given iterable, then yield others."
    for item in iterable:
        if qte == 0:
            yield item
        else:
            qte -= 1


@Pipe
def dedup(iterable, key=lambda x: x):
    """Only yield unique items. Use a set to keep track of duplicate data."""
    seen = set()
    for item in iterable:
        dupkey = key(item)
        if dupkey not in seen:
            seen.add(dupkey)
            yield item


@Pipe
def uniq(iterable, key=lambda x: x):
    """Deduplicate consecutive duplicate values."""
    iterator = iter(iterable)
    try:
        prev = next(iterator)
    except StopIteration:
        return
    yield prev
    prevkey = key(prev)
    for item in iterator:
        itemkey = key(item)
        if itemkey != prevkey:
            yield item
        prevkey = itemkey


enumerate = Pipe(builtins.enumerate)


@Pipe
def permutations(iterable, r=None):
    # permutations('ABCD', 2) --> AB AC AD BA BC BD CA CB CD DA DB DC
    # permutations(range(3)) --> 012 021 102 120 201 210
    for x in itertools.permutations(iterable, r):
        yield x


@Pipe
def netcat(to_send, host, port):
    """Send and receive bytes over TCP."""
    with closing(socket.socket(socket.AF_INET, socket.SOCK_STREAM)) as s:
        s.connect((host, port))
        for data in to_send | traverse:
            s.send(data)
        while 1:
            data = s.recv(4096)
            if not data:
                break
            yield data


@Pipe
def traverse(args):
    if isinstance(args, (str, bytes)):
        yield args
        return
    for arg in args:
        try:
            yield from arg | traverse
        except TypeError:
            # not iterable --- output leaf
            yield arg


@Pipe
def tee(iterable):
    for item in iterable:
        sys.stdout.write(repr(item) + "\n")
        yield item


@Pipe
def select(iterable: Iterable[T], selector: Callable[[T], S]) -> "builtins.map[S]":
    return builtins.map(selector, iterable)


map = select
""


@Pipe
def where(iterable: Iterable[T], predicate: Callable[[T], bool]) -> Generator[T, None, None]:
    return (x for x in iterable if predicate(x))


filter = where


@Pipe
def take_while(iterable, predicate):
    return itertools.takewhile(predicate, iterable)


@Pipe
def skip_while(iterable, predicate):
    return itertools.dropwhile(predicate, iterable)


@Pipe
def groupby(iterable, keyfunc):
    return itertools.groupby(sorted(iterable, key=keyfunc), keyfunc)


@Pipe
def sort(iterable: Iterable[T], key=None, reverse=False) -> list[T]:  # pylint: disable=redefined-outer-name
    return sorted(iterable, key=key, reverse=reverse)


@Pipe
def reverse(iterable):
    return reversed(iterable)


@Pipe
def t(iterable, y):
    if hasattr(iterable, "__iter__") and not isinstance(iterable, str):
        return iterable + type(iterable)([y])
    return [iterable, y]


@Pipe
def transpose(iterable):
    return list(zip(*iterable))


@Pipe
def tolist(iterable: Iterable[T]) -> list[T]:
    return list(iterable)


@Pipe
def toset(iterable: Iterable[T]) -> set[T]:
    return set(iterable)


chain = Pipe(itertools.chain.from_iterable)
chain_with = Pipe(itertools.chain)
islice = Pipe(itertools.islice)
izip = Pipe(zip)

if __name__ == "__main__":
    import doctest

    doctest.testfile("README.md")
